/* FL - A file (directory) listing  program.                         */

/* SUGS is a set of utilities for DOS that I developed during the    */
/* last few years.  Some of the programs were developed to provide   */
/* utilities missing in native DOS and others were developed just    */
/* for fun.  I used a couple of these programs all the time.  Maybe  */
/* you will find one or two of the programs useful in your work...   */
/*                                                                   */
/*       Hale Landis                                                 */
/*       PO Box 205                                                  */
/*       Louisville, CO 80027                                        */
/*                                                                   */
/* NOTE !  These programs are provided "as is" and you can't hold    */
/* me, Hale Landis, responsible for anything that happens to you or  */
/* your data or your system if you use them.  You may make all the   */
/* copies of the programs you need.  You may pass them on to your    */
/* friends (and enemies).  You may NOT charge anyone for a copy of   */
/* these programs (with the following exception: if someone insists  */
/* on paying you, you must send me 359.9472618 times the amount you  */
/* receive).  Don't use these programs in that product you are about */
/* to starting selling (the product that is going to make you as     */
/* rich and famous as IBM) without cutting me in on the action !     */
/* AND DON'T YOU DARE PUT THEM ON A BULLETIN BOARD !                 */

/* Compile with Lattice C 3.10 using small memory model.             */

#include "stdio.h"
#include "dos.h"

#define DEFSORT  'N'         /* default sort sequence          */
#define DOSFC7   7           /* DOS Function Call 7            */
#define MAXENT   512         /* max sort entries allowed       */
#define SCRNSIZE 24          /* number lines on console screen */

unsigned char csp[128];               /* complete search pattern        */
unsigned char drv[3];                 /* complete drive spec            */
unsigned char fspec[128] = "*.*";     /* complete file spec             */
unsigned char filetime[8];            /* unpacked file date/time array  */
unsigned char listopt = ' ';          /* list option                    */
unsigned char parm;                   /* parameter character            */
unsigned char path[128];              /* complete path from root dir    */
unsigned char *ptrparm;               /* pointer to parameter           */
unsigned char sortseq = ' ';          /* sort sequence                  */
unsigned char s1[128];                /* work area                      */
unsigned char s2[128];                /* work area                      */
unsigned char s3[128];                /* work area                      */

int f;                       /* 808x flags                     */
int lc;                      /* line count                     */
int listfile;                /* list file flag                 */
int loop;                    /* loop control                   */
int naster, ncolon, nperiod; /* character counters             */
int ndx1, ndx2, ndx3;        /* array indexes                  */
int nentry;                  /* number of entries to sort      */
int nfiles,ndirs;            /* number files and directories   */
int reverse;                 /* reverse option flag            */
int pause;                   /* pause option flag              */
int sai[MAXENT];             /* sort array index               */
int widecntr;                /* wide display counter           */
int widedis;                 /* wide display option flag       */
int w1, w2;                  /* work                           */

long int fsize,nbytes;       /* file size and total file size  */

struct FILEINFO dta;         /* see dos.h */

struct DIRENT                /* directory entry */
   {
   unsigned char sort[16];   /* sort field */
   unsigned char ne[12];     /* name and ext plus \x00 */
   unsigned char attr;       /* attribute */
   long int size;            /* file size */
   int dateyy;               /* file date year */
   int datemm;               /* file date month */
   int datedd;               /* file date day */
   int timehh;               /* file time hours */
   int timemm;               /* file time minutes */
   int timess;               /* file time seconds */
   };
struct DIRENT dir[MAXENT];   /* directory array */

#include "sugs.h"            /* include sugs special functions */

/********************************************************************/

void badparm(p)

unsigned char *p;

{

/* display bad parameter message */

fprintf(stderr,"\n\x07Invalid parameter (%s) ignored !\n",p);

}

/********************************************************************/

void chkpause()

{

/* if pause parameter is active, pause every SCRNSIZE output lines */

if ( pause )
   {
   lc++;
   if ( lc % SCRNSIZE == 0 )
      {
      cprintf("Strike a key when ready . . .");
      if ( bdos(DOSFC7) == 0 ) bdos(DOSFC7);
      cprintf("\r                             \r");
      }
   }

}

/********************************************************************/

void build()

{

/* determine the drive and path (from root) to use */
/* and get the complete file spec to use           */
/* output is: drv   = "d:"                         */
/*            path  = "\path-from-root\"           */
/*            fspec = "name.ext"  (primary)        */
/*            s3    = "name.ext"  (secondary)      */
/*            s2    = "nameext"   (no period)      */



/* if "d:" in fspec, remove it from fspec. */
/* if no "d:" in fspec, get default drive. */
if ( strlen(fspec) > 1 && fspec[1] == ':' )
   {
   drv[0] = fspec[0];
   strcpy(s1,&fspec[2]);
   strcpy(fspec,s1);
   if ( drv[0] < 'A' || drv[0] > 'Z' ) drv[0] = ' ';
   }
else
   {
   drv[0] = getdsk() + 'A';
   }
drv[1] = ':';
drv[2] = 0x00;

/* get default path for drive */
if ( drv[0] != ' ' )
   {
   s1[0] = '\\';
   if ( getcd(drv[0] - 'A' + 1,&s1[1]) != 0 ) drv[0] = ' ';
   }
if ( drv[0] == ' ' )
   {
   fprintf(stderr,"\n\x07Invalid drive specified !\n");
   exit(1);
   }
w1 = strlen(s1);
if ( s1[w1-1] != '\\' && s1[w1-1] != '/' )
   {
   s1[w1] = '\\';
   s1[w1+1] = 0x00;
   }

/* remove any path from fspec. */
w1 = 0;
w2 = -1;
while ( fspec[w1] != 0x00 )
   {
   if ( fspec[w1] == '\\' || fspec[w1] == '/' ) w2 = w1;
   w1++;
   }
if ( w2 >= 0 )
   {
   strcpy(s3,&fspec[w2+1]);
   fspec[w2+1] = 0x00;
   strcpy(s2,fspec);
   strcpy(fspec,s3);
   }

/* if (s2) is null, path is (s1).                  */
/* if (s2) starts with a '\' or '/', path is (s2). */
/* otherwise, path is (s1+s2).                     */
path[0] = 0x00;
if ( s2[0] == 0x00 ) strcpy(path,s1);
if ( s2[0] == '\\' || s2[0] == '/' ) strcpy(path,s2);
if ( path[0] == 0x00 )
   {
   strcpy(path,s1);
   strcat(path,s2);
   }

/* check for funny characters in path */
w1 = 0;
while ( path[w1] != 0x00 )
   {
   if ( path[w1] == ':' || path[w1] == '*' || path[w1] == '?' )
      {
      fprintf(stderr,"\n\x07Invalid path specified !\n");
      exit(1);
      }
   w1++;
   }

/* if fspec is null, change it to '*.*' */
if ( fspec[0] == 0x00 ) strcpy(fspec,"*.*");

/* save user file spec */
strcpy(s3,fspec);

/* if fspec starts with a '.', put a '*' in front */
if ( fspec[0] == '.' )
   {
   s1[0] = '*';
   s1[1] = 0x00;
   strcat(s1,fspec);
   strcpy(fspec,s1);
   }

/* if no '.' in fspec, add '.*' to fspec */
w1 = 0;
while ( fspec[w1] != 0x00 && fspec[w1] != '.' ) w1++;
if ( fspec[w1] != '.' ) strcat(fspec,".*");

/* check for funny characters in file spec */
w1 = 0;
ncolon = 0;
nperiod = 0;
while ( fspec[w1] != 0x00 )
   {
   if ( fspec[w1] == ':' ) ncolon++;
   if ( fspec[w1] == '.' ) nperiod++;
   w1++;
   }
if ( ncolon > 0 || nperiod > 1 )
   {
   fprintf(stderr,"\n\x07Invalid file specified !\n");
   exit(1);
   }

/* copy s3 to s2 without period */
strcpy(s2,"           ");
w1 = 0;
w2 = 0;
while ( w1 < 12 && w2 < 11 && s3[w1] != 0x00 )
   {
   if ( s3[w1] == '.' )
      w2 = 8;
   else
      s2[w2++] = s3[w1];
   w1++;
   }
}

/********************************************************************/

void getdir()

{

/* do the search and build the directory/file array */

ndx1 = 0;
loop = 1;

if ( dfind(&dta,csp,0x16) != 0 ) loop = 0;
if ( loop == 0 && _OSERR != 18 )
   {
   fprintf(stderr,"\n\x07Drive or path does not exist !\n");
   exit(1);
   }

while ( loop )
   {
   if ( ndx1 >= MAXENT )
      {
      fprintf(stderr,"\n\x07Directory size exceeds capacity !\n");
      exit(1);
      }
   for ( w1 = 0; w1 < 11; w1++ ) dir[ndx1].ne[w1] = ' ';
   if ( ( dta.attr & 0x10 ) && ( dta.name[0] == '.' ) )
      {
      dir[ndx1].ne[0] = '.';
      if ( dta.name[1] == '.' ) dir[ndx1].ne[1] = '.';
      }
   else
      {
      w1 = 0;
      w2 = 0;
      while ( w2 < 11 && dta.name[w1] != 0x00 )
         {
         if ( dta.name[w1] == '.' )
            w2 = 8;
         else
            dir[ndx1].ne[w2++] = dta.name[w1];
         w1++;
         }
      }
   dir[ndx1].ne[11] = 0x00;
   dir[ndx1].attr = dta.attr;
   dir[ndx1].size = dta.size;
   ftunpk(dta.time,filetime);
   dir[ndx1].dateyy = 80 + filetime[0];
   dir[ndx1].datemm = filetime[1];
   dir[ndx1].datedd = filetime[2];
   dir[ndx1].timehh = filetime[3];
   dir[ndx1].timemm = filetime[4];
   dir[ndx1].timess = filetime[5];
   sai[ndx1] = ndx1;
   ndx1++;
   if ( dnext(&dta) != 0 ) loop = 0;
   }
nentry = ndx1;

}

/********************************************************************/

void flist1()

{

/* add the sort field to the dir array */

for ( w1 = 0; w1 < nentry; w1++ )
   {
   for ( w2 = 0; w2 < 16; w2++ ) dir[w1].sort[w2] = ' ';
   if ( sortseq == 'N' )
      if ( dir[w1].attr & 0x10 )
         dir[w1].sort[0] = '1';
      else
         dir[w1].sort[0] = '2';
      for ( w2 = 0; w2 < 8; w2++ ) dir[w1].sort[1+w2] = dir[w1].ne[0+w2];
      for ( w2 = 0; w2 < 3; w2++ ) dir[w1].sort[9+w2] = dir[w1].ne[8+w2];
   if ( sortseq == 'E' )
      {
      if ( dir[w1].attr & 0x10 )
         dir[w1].sort[0] = '1';
      else
         dir[w1].sort[0] = '2';
      for ( w2 = 0; w2 < 3; w2++ ) dir[w1].sort[1+w2] = dir[w1].ne[8+w2];
      for ( w2 = 0; w2 < 8; w2++ ) dir[w1].sort[4+w2] = dir[w1].ne[0+w2];
      }
   if ( sortseq == 'S' )
      {
      sprintf(&dir[w1].sort[0],"%08ld",dir[w1].size);
      dir[w1].sort[8] = ' ';
      }
   if ( sortseq == 'D' )
      {
      sprintf(&dir[w1].sort[0],
              "%02d%02d%02d %02d%02d%02d",
              dir[w1].dateyy,dir[w1].datemm,dir[w1].datedd,
              dir[w1].timehh,dir[w1].timemm,dir[w1].timess);
      dir[w1].sort[13] = ' ';
      }
   }

}

/********************************************************************/

void sortdir()

{

/* sort the directory data */

for ( ndx1 = nentry - 1; ndx1 > 0; ndx1-- )
   {
   for ( ndx2 = 0; ndx2 < ndx1; ndx2++ )
      {
      ndx3 = strcmp(&dir[sai[ndx2]].sort[0],&dir[sai[ndx2+1]].sort[0]);
      if ( ndx3 > 0 )
         {
         ndx3 = sai[ndx2+0];
         sai[ndx2+0] = sai[ndx2+1];
         sai[ndx2+1] = ndx3;
         }
      }
   }

}

/********************************************************************/

void flist2()

{

/* display the sorted directory data on stdout. */

cprintf("\nDirectory of %s\n\n",csp);
chkpause();
chkpause();
chkpause();
cbrkerr(XIT_INS);
if ( reverse )
   ndx1 = nentry - 1;
else
   ndx1 = 0;
while ( ( reverse && ndx1 >= 0 ) || ( ! reverse && ndx1 < nentry ) )
   {
   if ( cbrkef == 0 ) cbrkerr(XIT_QERR);
   ndx2 = sai[ndx1];
   if ( listopt == ' ' )
      listfile = 1;
   else
      listfile = 0;
   if ( listopt == 'D'
        && ( ( dir[ndx2].attr & 0x10 ) != 0 ) ) listfile = 1;
   if ( listopt == 'F'
        && ( ( dir[ndx2].attr & 0x10 ) == 0 ) ) listfile = 1;
   if ( listopt == 'X'
        && ( ( dir[ndx2].attr & 0x10 ) == 0 ) )
      {
      if ( strncmp(&dir[ndx2].ne[8],"BAT",3) == 0 ) listfile = 1;
      if ( strncmp(&dir[ndx2].ne[8],"COM",3) == 0 ) listfile = 1;
      if ( strncmp(&dir[ndx2].ne[8],"EXE",3) == 0 ) listfile = 1;
      }
   if ( listfile )
      {
      if ( dir[ndx2].attr == 0x10 )
         {
         ndirs++;
         strcpy(s2,"<DIR>   ");
         }
      else
         {
         nfiles++;
         sprintf(s2,"%8ld",dir[ndx2].size);
         s2[8] = 0x00;
         nbytes = nbytes + dir[ndx2].size;
         }
      if ( cbrkef == 0 )
         {
         for ( w1 = 0; w1 < 8; w1++ ) s1[w1] = dir[ndx2].ne[w1];
         s1[8] = ' ';
         s1[9] = ' ';
         for ( w1 = 0; w1 < 3; w1++ ) s1[10+w1] = dir[ndx2].ne[8+w1];
         s1[13] = 0x00;
         if ( widedis )
            {
            if ( widecntr % 4 > 0 )
               cprintf("  ||  ");
            cprintf("%s",s1);
            widecntr++;
            if ( widecntr % 4 == 0 )
               {
               cprintf("\n");
               chkpause();
               }
            }
         else
            {
            if ( dir[ndx2].attr & 0x01 ) s3[0] = 'R'; else s3[0] = ' ';
            if ( dir[ndx2].attr & 0x02 ) s3[1] = 'H'; else s3[1] = ' ';
            if ( dir[ndx2].attr & 0x04 ) s3[2] = 'S'; else s3[2] = ' ';
            if ( dir[ndx2].attr & 0x20 ) s3[3] = 'A'; else s3[3] = ' ';
            s3[4] = 0x00;
            cprintf("%s  %s  %02d/%02d/%02d  %02d:%02d:%02d  %s\n",
                    s1,
                    s2,
                    dir[ndx2].datemm,dir[ndx2].datedd,dir[ndx2].dateyy,
                    dir[ndx2].timehh,dir[ndx2].timemm,dir[ndx2].timess,
                    s3);
            chkpause();
            }
         }
      }
   if ( reverse )
      ndx1--;
   else
      ndx1++;
   }
if ( widedis && ( widecntr % 4 != 0 ) )
   {
   cprintf("\n");
   chkpause();
   }
cbrkerr(XIT_RMV);

if ( ( nfiles + ndirs ) > 0 )
   {
   cprintf("\n");
   chkpause();
   }
cprintf("%d file(s) use %ld bytes (%d subdirectories)\n",
        nfiles,nbytes,ndirs);
chkpause();

}

/********************************************************************/

void main(argc,argv)

int argc;
unsigned char *argv[];

{

/* main fucntion */

int argnum, argpat, argsiz;

switchar();

argpat = 0;
for ( argnum = 1; argnum < argc; argnum++ )
   {
   ptrparm = argv[argnum];
   argsiz = strlen(ptrparm);
   if ( *ptrparm != schar )
      {
      if ( argpat )
         badparm(ptrparm);
      else
         argpat = argnum;
      }
   else
      {
      if ( argsiz > 1 )
         parm = toupper( *(ptrparm + 1) );
      else
         parm = ' ';
      switch( parm )
         {
         case 'D':
         case 'E':
         case 'N':
         case 'S':
            if ( argsiz == 2 && sortseq == ' ' )
               sortseq = parm;
            else
               badparm(ptrparm);
            break;
         case 'L':
            if ( argsiz == 3 && listopt == ' ' )
               {
               listopt = toupper( *(ptrparm + 2) );
               if ( listopt != 'D' && listopt != 'F' && listopt != 'X' )
                  {
                  badparm(ptrparm);
                  listopt = ' ';
                  }
               }
            else
               badparm(ptrparm);
            break;
         case 'P':
            if ( argsiz == 2 && ! pause )
               pause = 1;
            else
               badparm(ptrparm);
            break;
         case 'R':
            if ( argsiz == 2 && ! reverse )
               reverse = 1;
            else
               badparm(ptrparm);
            break;
         case 'W':
            if ( argsiz == 2 && ! widedis )
               widedis = 1;
            else
               badparm(ptrparm);
            break;
         default:
            badparm(ptrparm);
            break;
         }
      }
   }

if ( argpat ) strcpy(fspec,argv[argpat]);
w1 = 0;
while ( fspec[w1] != 0x00 )
   {
   fspec[w1] = toupper( fspec[w1] );
   w1++;
   }

if ( sortseq == ' ' ) sortseq = DEFSORT;

build();
strcpy(csp,drv);
strcat(csp,path);
strcat(csp,fspec);

getdir();
if ( nentry == 1 && ( dir[0].attr & 0x10 )
     && ( strcmp(s2,dir[0].ne) == 0 )
   )
   {
   strcpy(csp,drv);
   strcat(csp,path);
   strcat(csp,s3);
   strcat(csp,"\\*.*");
   nentry = 0;
   getdir();
   }

flist1();

sortdir();

flist2();

exit(0);

}

/* end FL.C source file */
